import cx from 'clsx';
import { useAtom } from 'jotai';
import { isEmpty, isEqual } from 'lodash-es';
import { useCallback, useMemo } from 'react';
import { Link, useNavigate, useParams } from 'react-router-dom';
import { ActionIcon } from '../../components/ActionIcon';
import { Badge } from '../../components/Badge';
import { SchemeSign } from '../../components/SchemeSign';
import { activeSchemeAtom, useRemoveScheme, useSchemeList } from '../../stores/schemes';
import styles from './SchemeList.module.css';

function OperateButtons() {
  return (
    <div className={styles.operates_buttons}>
      <Link to="new" className="button">
        New Scheme
      </Link>
    </div>
  );
}

type SchemeItemProps = {
  item: ReturnType<typeof useSchemeList>[number];
};

function SchemeItem({ item }: SchemeItemProps) {
  const navParams = useParams();
  const navigate = useNavigate();
  const [activedScheme, setActiveScheme] = useAtom(activeSchemeAtom);
  const removeScheme = useRemoveScheme(item.id);
  const isActived = useMemo(() => isEqual(activedScheme, item.id), [activedScheme, item.id]);
  const isSelected = useMemo(() => isEqual(navParams['id'], item.id), [navParams, item.id]);
  const handleActiveScheme = useCallback(() => {
    setActiveScheme((prev) => (prev ? null : item.id));
  }, [item]);
  const handleRemoveScheme = useCallback(() => {
    removeScheme();
    if (isActived) {
      setActiveScheme(null);
    }
    navigate(-1);
  }, [item, isActived]);

  return (
    <div
      className={cx(styles.scheme_item, isSelected && styles.selected)}
      onClick={() => navigate(item.id)}>
      <div className={styles.name}>{item.name}</div>
      <div className={styles.status}>
        <SchemeSign scheme={item.type} short />
        {isActived && <Badge extendClassName={styles.active_badge}>ACTIVE</Badge>}
        <div className="spacer"></div>
        {isSelected && (
          <>
            <ActionIcon
              icon="tabler:trash"
              extendClassName={styles.delete_btn}
              onClick={handleRemoveScheme}
            />
            <ActionIcon
              icon="tabler:checkbox"
              extendClassName={cx(styles.active_btn, isActived && styles.deactive)}
              onClick={handleActiveScheme}
            />
          </>
        )}
      </div>
    </div>
  );
}

function SchemesItems() {
  const schemes = useSchemeList();

  return (
    <div className={styles.scheme_items}>
      {isEmpty(schemes) && <div className={styles.empty_prompt}>Create a scheme first.</div>}
      {schemes.map((item) => (
        <SchemeItem key={item.id} item={item} />
      ))}
    </div>
  );
}

export function SchemeList() {
  return (
    <div className={styles.scheme_list}>
      <OperateButtons />
      <SchemesItems />
    </div>
  );
}
